\begindata{text,538374540}
\textdsversion{12}
\template{default}
\define{global
}
\majorheading{Link}

\center{A File Reference Inset}


\quotation{\center{Abstract}

Link is an ATK inset that implments a file referencing tool.  When a user 
presses the left button on a link, the referenced file will be brought up in a 
window and the focus and mouse pointer will be set to that window.

}
\heading{User Documentation}

For end user documentation see 
\begindata{link,539282952}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Help.d
0
0
\begindata{link,539282952}
Datastream version: 2
LinkHelp
2
andy12b
black
white
\enddata{link,539282952}
\enddata{link,539282952}
\view{linkv,539282952,762,0,0}.


\heading{Hypermedia Developer Documentation}

For developer documentation see 
\begindata{link,539353096}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Developr.d
0
0
\begindata{link,539353096}
Datastream version: 2
Link Developer
2
andy12b
black
white
\enddata{link,539353096}
\enddata{link,539353096}
\view{linkv,539353096,763,0,0}.


\heading{Link Proctable Entries}

To know what Procedure Table entries Link defines for use in menus and 
keybinding, see 
\begindata{link,539283080}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Procs.d
0
0
\begindata{link,539283080}
Datastream version: 2
LinkProcs
2
andy12b
black
white
\enddata{link,539283080}
\enddata{link,539283080}
\view{linkv,539283080,764,0,0}.


\heading{Link Preferences}

If you would like to modify the way Link looks, see 
\begindata{link,539353224}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Prefs.d
0
0
\begindata{link,539353224}
Datastream version: 2
LinkPreferences
2
andy12b
black
white
\enddata{link,539353224}
\enddata{link,539353224}
\view{linkview,539353224,765,0,0}.


\heading{Link Class Header}

To see the link header file see 
\begindata{link,539283208}
Datastream version: 3
/usr/andrew/include/atk/link.ch
0
0
\begindata{link,539283208}
Datastream version: 2
link.ch
2
andy12b
black
white
\enddata{link,539283208}
\enddata{link,539283208}
\view{linkv,539283208,766,0,0}.

To see the linkview header file see 
\begindata{link,539283336}
Datastream version: 3
/usr/andrew/include/atk/linkv.ch
0
0
\begindata{link,539283336}
Datastream version: 2
linkv.ch
2
andy12b
black
white
\enddata{link,539283336}
\enddata{link,539283336}
\view{linkv,539283336,767,0,0}.




\begindata{bp,537558784}
\enddata{bp,537558784}
\view{bpv,537558784,769,0,0}
Copyright 1992 Carnegie Mellon University and IBM.  All rights reserved.

\smaller{\smaller{$Disclaimer: 

Permission to use, copy, modify, and distribute this software and its 

documentation for any purpose is hereby granted without fee, 

provided that the above copyright notice appear in all copies and that 

both that copyright notice, this permission notice, and the following 

disclaimer appear in supporting documentation, and that the names of 

IBM, Carnegie Mellon University, and other copyright holders, not be 

used in advertising or publicity pertaining to distribution of the software 

without specific, written prior permission.



IBM, CARNEGIE MELLON UNIVERSITY, AND THE OTHER COPYRIGHT HOLDERS 

DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING 

ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT 

SHALL IBM, CARNEGIE MELLON UNIVERSITY, OR ANY OTHER COPYRIGHT HOLDER 

BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY 

DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 

WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 

ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 

OF THIS SOFTWARE.

 $

}}\enddata{text,538374540}
