\begindata{text,539034464}
\textdsversion{12}
\template{default}
\define{global
}
\majorheading{Link User Documentation

}\center{How to use Linked Documents}


\quotation{\center{Abstract}

As a link user, you can easily navigate a set of Linked documents by using 
your left mouse button.

}
\heading{What is a Link?}

A link is an ATK Inset.  Hypermedia Developers place Links in documents to 
help you locate additional information (if you'd like information on making 
your own Linked documents, see 
\begindata{link,539283336}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Developr.d
0
0
\begindata{link,539283336}
Datastream version: 2
Link Developer
2
andysans8b
black
white
\enddata{link,539283336}
\enddata{link,539283336}
\view{linkv,539283336,770,0,0}).  By default, a link looks like a pushbutton. 
 However, you may set your "\typewriter{linkstyle}" preference to "0", which 
makes a link look like regular text, typically in \bold{Boldface} (although it 
may not be), (see 
\begindata{link,539283208}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Prefs.d
0
0
\begindata{link,539283208}
Datastream version: 2
Link Preferences
2
andysans8b
black
white
\enddata{link,539283208}
\enddata{link,539283208}
\view{linkview,539283208,771,0,0}).  The Hypermedia Developer will usually 
give some kind of contextual indication that there is a link here, like "see 
such-and-such".  Also, the mouse cursor will change from a North-West arrow to 
a crosshairs sight cursor.


\heading{How do I use a Link?}

If you would like to follow the Link, just press your left mouse button over 
the link inset (i.e., anywhere in the region where the target cursor is 
visible).  The Link inset will then bring up a window with the referenced 
document in it, and your mouse pointer will be warped to that window.  The 
original source document will remain in its window.


\heading{What else should I know?}

\description{To remove a window brought up by a link, remember to use "^X^D" 
(delete window), and not "^X^C" (quit application).


When a Link has the input focus, typing to it will cause the "Unkown command" 
message to appear.  If you want to type in your document, simply use the mouse 
to select a new insertion point.


Sometimes a Link inset won't have a target document.  In this case, pressing 
on the link will cause the message "No link." to be displayed.  It will also 
give the input focus to the link (see previous note).


There is a Link menu, which is meant for Hypermedia Developers.  You should 
ignore it.  If you accidentally select one of these menu items, "^G" should 
get you out of it.

}
\heading{Where can I find out more?}

The overview document for Link is 
\begindata{link,539353224}
Datastream version: 3
/usr/andrew/doc/atk/linkinset/Link.d
0
0
\begindata{link,539353224}
Datastream version: 2
Link
2
andy12b
black
white
\enddata{link,539353224}
\enddata{link,539353224}
\view{linkv,539353224,772,0,0}.


\begindata{bp,537558784}
\enddata{bp,537558784}
\view{bpv,537558784,774,0,0}
Copyright 1992 Carnegie Mellon University and IBM.  All rights reserved.

\smaller{\smaller{$Disclaimer: 

Permission to use, copy, modify, and distribute this software and its 

documentation for any purpose is hereby granted without fee, 

provided that the above copyright notice appear in all copies and that 

both that copyright notice, this permission notice, and the following 

disclaimer appear in supporting documentation, and that the names of 

IBM, Carnegie Mellon University, and other copyright holders, not be 

used in advertising or publicity pertaining to distribution of the software 

without specific, written prior permission.



IBM, CARNEGIE MELLON UNIVERSITY, AND THE OTHER COPYRIGHT HOLDERS 

DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING 

ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT 

SHALL IBM, CARNEGIE MELLON UNIVERSITY, OR ANY OTHER COPYRIGHT HOLDER 

BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY 

DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 

WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 

ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 

OF THIS SOFTWARE.

 $

}}\enddata{text,539034464}
